@@ -92,8 +92,9 @@ from oletools import rtfobj
 # 2018-01-05       CH: - add CSV
 # 2018-01-11       PL: - fixed issue #242 (apply unquote to fldSimple tags)
 # 2018-01-10       CH: - add single-xml files (Word 2003/2007+ / Excel 2003)
+# 2018-03-21       CH: - added detection for various CSV formulas (issue #259)
 
-__version__ = '0.52'
+__version__ = '0.53'
 
 # -----------------------------------------------------------------------------
 # TODO: field codes can be in headers/footers/comments - parse these
@@ -565,20 +566,21 @@ def process_docx(filepath, field_filter_mode=None):
             continue
 
         # have a TAG_W_P
-        elem = None
         for curr_elem in subs:
             # check if w:r; parse children to pull out first FLDCHAR/INSTRTEXT
+            elem = None
             if curr_elem.tag in TAG_W_R:
                 for child in curr_elem:
                     if child.tag in TAG_W_FLDCHAR or \
                             child.tag in TAG_W_INSTRTEXT:
                         elem = child
                         break
+                if elem is None:
+                    continue   # no fldchar or instrtext in this w:r
             else:
                 elem = curr_elem
             if elem is None:
-                logging.warning('this should be an error condition')
-                continue
+                raise BadOOXML(filepath, 'Got "None"-Element from iter_xml')
 
             # check if FLDCHARTYPE and whether "begin" or "end" tag
             attrib_type = elem.attrib.get(ATTR_W_FLDCHARTYPE[0]) or \
@@ -832,7 +834,8 @@ def process_rtf(file_handle, field_filter_mode=None):
 CSV_SMALL_THRESH = 1024
 
 # format of dde link: program-name | arguments ! unimportant
-CSV_DDE_FORMAT = re.compile(r'\s*=(.+)\|(.+)!(.*)\s*')
+# can be enclosed in "", prefixed with + or = or - or cmds like @SUM(...)
+CSV_DDE_FORMAT = re.compile(r'\s*"?[=+-@](.+)\|(.+)!(.*)\s*')
 
 # allowed delimiters (python sniffer would use nearly any char). Taken from
 # https://data-gov.tw.rpi.edu/wiki/CSV_files_use_delimiters_other_than_commas
