@@ -14,7 +14,7 @@ http://www.decalage.info/python/oletools
 
 #=== LICENSE ==================================================================
 
-# oledir is copyright (c) 2015-2017 Philippe Lagadec (http://www.decalage.info)
+# oledir is copyright (c) 2015-2018 Philippe Lagadec (http://www.decalage.info)
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without modification,
@@ -48,8 +48,10 @@ from __future__ import print_function
 # 2016-08-09       PL: - fixed issue #77 (imports from thirdparty dir)
 # 2017-03-08 v0.51 PL: - fixed absolute imports, added optparse
 #                      - added support for zip files and wildcards
+# 2018-04-11 v0.53 PL: - added table displaying storage tree and CLSIDs
+# 2018-04-13       PL: - moved KNOWN_CLSIDS to common.clsid
 
-__version__ = '0.51'
+__version__ = '0.53'
 
 #------------------------------------------------------------------------------
 # TODO:
@@ -88,7 +90,7 @@ if os.name == 'nt':
 from oletools.thirdparty import olefile
 from oletools.thirdparty.tablestream import tablestream
 from oletools.thirdparty.xglob import xglob
-
+from oletools.common.clsid import KNOWN_CLSIDS
 
 # === CONSTANTS ==============================================================
 
@@ -105,7 +107,7 @@ STORAGE_NAMES = {
 
 STORAGE_COLORS = {
     olefile.STGTY_EMPTY:     'green',
-    olefile.STGTY_STORAGE:   'blue',
+    olefile.STGTY_STORAGE:   'cyan',
     olefile.STGTY_STREAM:    'yellow',
     olefile.STGTY_LOCKBYTES: 'magenta',
     olefile.STGTY_PROPERTY:  'magenta',
@@ -127,6 +129,13 @@ def sid_display(sid):
     else:
         return sid
 
+def clsid_display(clsid):
+    if clsid in KNOWN_CLSIDS:
+        clsid += '\n%s' % KNOWN_CLSIDS[clsid]
+    color = 'yellow'
+    if 'CVE' in clsid:
+        color = 'red'
+    return (clsid, color)
 
 # === MAIN ===================================================================
 
@@ -227,10 +236,38 @@ def main():
             # t.add_row((id, status, entry_type, name, left, right, child, hex(d.isectStart), d.size))
             table.write_row((id, status, entry_type, name, left, right, child, '%X' % d.isectStart, d.size),
                 colors=(None, status_color, etype_color, None, None, None, None, None, None))
-            print(d.clsid)
+
+        table = tablestream.TableStream(column_width=[4, 28, 6, 38],
+            header_row=('id', 'Name', 'Size', 'CLSID'),
+            style=tablestream.TableStyleSlim)
+        rootname = ole.get_rootentry_name()
+        entry_id = 0
+        clsid = ole.root.clsid
+        clsid_text, clsid_color = clsid_display(clsid)
+        table.write_row((entry_id, rootname, '-', clsid_text),
+                        colors=(None, 'cyan', None, clsid_color))
+        for entry in sorted(ole.listdir(storages=True)):
+            name = entry[-1]
+            # handle non-printable chars using repr(), remove quotes:
+            name = repr(name)[1:-1]
+            name_color = None
+            if ole.get_type(entry) in (olefile.STGTY_STORAGE, olefile.STGTY_ROOT):
+                name_color = 'cyan'
+            indented_name = '  '*(len(entry)-1) + name
+            entry_id = ole._find(entry)
+            try:
+                size = ole.get_size(entry)
+            except:
+                size = '-'
+            clsid = ole.getclsid(entry)
+            clsid_text, clsid_color = clsid_display(clsid)
+            table.write_row((entry_id, indented_name, size, clsid_text),
+                            colors=(None, name_color, None, clsid_color))
+
+
         ole.close()
         # print t
 
 
 if __name__ == '__main__':
-    main()
\ No newline at end of file
+    main()
